package jalview.ws.jws2;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashSet;
import java.util.Vector;

import javax.swing.JMenu;
import javax.swing.JMenuItem;

import org.apache.log4j.Level;

import jalview.bin.Cache;
import jalview.datamodel.AlignmentView;
import jalview.gui.AlignFrame;
import jalview.ws.WSMenuEntryProviderI;
import compbio.data.msa.MsaWS;
import compbio.ws.client.Jws2Base;
import compbio.ws.client.Jws2Base.Services;

/**
 * discoverer for jws2 services. Follows the lightweight service discoverer
 * pattern (archetyped by EnfinEnvision2OneWay)
 * 
 * @author JimP
 * 
 */
public class Jws2Discoverer implements Runnable, WSMenuEntryProviderI
{
  compbio.data.msa.MsaWS service;
  boolean running=false;
  @Override
  public void run()
  {
    if (running)
    {
      return;
    }
    running=true;
//    Cache.initLogger();
//    Cache.log.setLevel(Level.DEBUG);
    // TODO: Document and PACK JWS2
    String jwsservers = Cache.getDefault("JWS2HOSTURLS",
            "http://webservices.compbio.dundee.ac.uk:8084/jws2");
    try
    {
      if (Jws2Base.validURL(jwsservers))
      {
        // look for services
        for (Services srv : Jws2Base.Services.values())
        {
          MsaWS service = null;
          try
          {
            service = Jws2Base.connect(jwsservers, srv);
          } catch (Exception e)
          {
          }
          ;
          if (service != null)
          {
            addService(jwsservers, srv, service);
          }
        }

      }
      else
      {
        Cache.log.info("Ignoring invalid Jws2 service url " + jwsservers);
      }
    } catch (Exception e)
    {
      Cache.log.warn("Exception when discovering Jws2 services.", e);
    } catch (Error e)
    {
      Cache.log.error("Exception when discovering Jws2 services.", e);
    }
    running=false;

  }

  /**
   * record this service endpoint so we can use it
   * 
   * @param jwsservers
   * @param srv
   * @param service2
   */
  private void addService(String jwsservers, Services srv, MsaWS service2)
  {
    Cache.log.info("Discovered service: " + jwsservers + " "
            + srv.toString());
    if (services==null) {
      services = new Vector<Jws2Instance>();
    }
    services.add(new Jws2Instance(jwsservers, "Align with "
            + srv.toString(), service2));
  }

  public class Jws2Instance
  {
    String hosturl;

    String serviceType;

    MsaWS service;

    public Jws2Instance(String hosturl, String serviceType, MsaWS service)
    {
      super();
      this.hosturl = hosturl;
      this.serviceType = serviceType;
      this.service = service;
    }

  };

  /**
   * holds list of services.
   */
  Vector<Jws2Instance> services;

  @Override
  public void attachWSMenuEntry(JMenu wsmenu, final AlignFrame alignFrame)
  {
    if (running || services==null || services.size() == 0)
    {
      return;
    }
    /**
     * eventually, JWS2 services will appear under the same align/etc submenus.
     * for moment we keep them separate.
     */
    JMenu jws2 = new JMenu("JWS2 Alignment");
    MsaWSClient msacl = new MsaWSClient();
    for (final Jws2Instance service : services)
    {
      msacl.attachWSMenuEntry(jws2, service, alignFrame);
      /*JMenuItem sitem = new JMenuItem(service.serviceType);
      sitem.setToolTipText("Hosted at " + service.hosturl);
      sitem.addActionListener(new ActionListener()
      {

        @Override
        public void actionPerformed(ActionEvent e)
        {
          AlignmentView msa = alignFrame.gatherSequencesForAlignment();
          MsaWSClient client = new MsaWSClient(service,
                  "JWS2 Alignment of " + alignFrame.getTitle(), msa, false,
                  true, alignFrame.getViewport().getAlignment().getDataset(),
                  alignFrame);
        }
      });*/
    }
    if (services.size()>0)
    {
      wsmenu.add(jws2);
    }
    
  }
  public static void main(String[] args)
  {
    Thread runner = new Thread(getDiscoverer());
    runner.start();
    while (runner.isAlive())
    {
      try
      {
        Thread.sleep(50);
      } catch (InterruptedException e)
      {
      }
      ;
    }
  }
  private static Jws2Discoverer discoverer;
  public static Jws2Discoverer getDiscoverer()
  {
    if (discoverer==null)
    {
      discoverer = new Jws2Discoverer();
    }
    return discoverer;
  }

  public boolean hasServices()
  {
    // TODO Auto-generated method stub
    return services!=null && services.size()>0;
  }
  
}
